@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -27,7 +27,7 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
@@ -50,8 +50,8 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -88,7 +88,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h 13888 2009-06-24 21:35:54Z fyodor $ */
+/* $Id: NmapOps.h 22066 2011-01-27 21:49:15Z david $ */
 
 #include "nmap.h"
 #include "global_structures.h"
@@ -115,7 +115,7 @@ class NmapOps {
   const struct timeval *getStartTime() { return &start_time; }
   // Number of milliseconds since getStartTime().  The current time is an
   // optional argument to avoid an extra gettimeofday() call.
-  int TimeSinceStartMS(struct timeval *now=NULL); 
+  int TimeSinceStartMS(const struct timeval *now=NULL); 
   struct in_addr v4source();
   const struct in_addr *v4sourceip();
 
@@ -182,7 +182,6 @@ class NmapOps {
   int spoofsource; /* -S used */
   int fastscan;
   char device[64];
-  int interactivemode;
   int ping_group_sz;
   int nogcc; /* Turn off group congestion control with --nogcc */
   int generate_random_ips; /* -iR option */
@@ -280,7 +279,6 @@ class NmapOps {
   int servicescan;
   int pingtype;
   int listscan;
-  int pingscan;
   int allowall;
   int fragscan; /* 0 or MTU (without IPv4 header size) */
   int ackscan;
@@ -299,6 +297,7 @@ class NmapOps {
   int windowscan;
   int xmasscan;
   int noresolve;
+  int noportscan;
   int append_output; /* Append to any output files rather than overwrite */
   FILE *logfd[LOG_NUM_FILES];
   FILE *nmap_stdout; /* Nmap standard output */
@@ -327,6 +326,7 @@ class NmapOps {
   int scriptversion;
   int scripttrace;
   int scriptupdatedb;
+  bool scripthelp;
   void chooseScripts(char* argument);
   std::vector<std::string> chosenScripts;
 #endif
