@@ -34,6 +34,8 @@ class Scan(object):
         self.start_date = None
         self.end_date = None
         self.hosts = []
+        self.pre_script_results = []
+        self.post_script_results = []
 
     def find_host(self, id):
         for host in self.hosts:
@@ -388,6 +390,42 @@ def format_banner(scan):
         parts.append(u"at %s" % scan.start_date.strftime("%Y-%m-%d %H:%M"))
     return u" ".join(parts)
 
+def print_script_result_diffs_text(title, script_results_a, script_results_b,
+    script_result_diffs, f = sys.stdout):
+    table = Table(u"*")
+    for sr_diff in script_result_diffs:
+        sr_diff.append_to_port_table(table)
+    if len(table) > 0:
+        print >> f
+        if len(script_results_b) == 0:
+            print >> f, u"-%s:" % title
+        elif len(script_results_a) == 0:
+            print >> f, u"+%s:" % title
+        else:
+            print >> f, u" %s:" % title
+        print >> f, table
+
+def script_result_diffs_to_dom_fragment(elem, script_results_a,
+    script_results_b, script_result_diffs, document):
+    if len(script_results_a) == 0 and len(script_results_b) == 0:
+        return document.createDocumentFragment()
+    elif len(script_results_b) == 0:
+        a_elem = document.createElement(u"a")
+        for sr in script_results_a:
+            elem.appendChild(sr.to_dom_fragment(document))
+        a_elem.appendChild(elem)
+        return a_elem
+    elif len(script_results_a) == 0:
+        b_elem = document.createElement(u"b")
+        for sr in script_results_b:
+            elem.appendChild(sr.to_dom_fragment(document))
+        b_elem.appendChild(elem)
+        return b_elem
+    else:
+        for sr_diff in script_result_diffs:
+            elem.appendChild(sr_diff.to_dom_fragment(document))
+        return elem
+
 class ScanDiff(object):
     """A complete diff of two scans. It is a container for two scans and a dict
     mapping hosts to HostDiffs."""
@@ -396,6 +434,8 @@ class ScanDiff(object):
         self.scan_a = scan_a
         self.scan_b = scan_b
         self.hosts = []
+        self.pre_script_result_diffs = []
+        self.post_script_result_diffs = []
         self.host_diffs = {}
 
         self.diff()
@@ -414,6 +454,9 @@ class ScanDiff(object):
                 self.hosts.append(host)
                 self.host_diffs[host] = h_diff
 
+        self.pre_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.pre_script_results, self.scan_b.pre_script_results)
+        self.post_script_result_diffs = ScriptResultDiff.diff_lists(self.scan_a.post_script_results, self.scan_b.post_script_results)
+
     def print_text(self, f = sys.stdout):
         """Print this diff in a human-readable text form."""
         banner_a = format_banner(self.scan_a)
@@ -424,12 +467,20 @@ class ScanDiff(object):
         else:
             print >> f, u" %s" % banner_a
 
+        print_script_result_diffs_text("Pre-scan script results",
+            self.scan_a.pre_script_results, self.scan_b.pre_script_results,
+            self.pre_script_result_diffs)
+
         for host in self.hosts:
-            print
+            print >> f
 
             h_diff = self.host_diffs[host]
             h_diff.print_text(f)
 
+        print_script_result_diffs_text("Post-scan script results",
+            self.scan_a.post_script_results, self.scan_b.post_script_results,
+            self.post_script_result_diffs)
+
     def print_xml(self, f = sys.stdout):
         impl = xml.dom.minidom.getDOMImplementation()
         document = impl.createDocument(None, u"nmapdiff", None)
@@ -438,11 +489,27 @@ class ScanDiff(object):
         scandiff_elem = document.createElement(u"scandiff")
         root.appendChild(scandiff_elem)
 
+        # prerule script changes.
+        if len(self.pre_script_result_diffs) > 0 or verbose:
+            prescript_elem = document.createElement(u"prescript")
+            scandiff_elem.appendChild(script_result_diffs_to_dom_fragment(
+                prescript_elem, self.scan_a.pre_script_results,
+                self.scan_b.pre_script_results, self.pre_script_result_diffs,
+                document))
+
         for host in self.hosts:
             h_diff = self.host_diffs[host]
             frag = h_diff.to_dom_fragment(document)
             scandiff_elem.appendChild(frag)
 
+        # postrule script changes.
+        if len(self.post_script_result_diffs) > 0 or verbose:
+            postscript_elem = document.createElement(u"postscript")
+            scandiff_elem.appendChild(script_result_diffs_to_dom_fragment(
+                postscript_elem, self.scan_a.post_script_results,
+                self.scan_b.post_script_results, self.post_script_result_diffs,
+                document))
+
         document.writexml(f, addindent = u"  ", newl = u"\n", encoding = "UTF-8")
         document.unlink()
 
@@ -587,18 +654,9 @@ class HostDiff(object):
                     for i in range(i1, i2):
                         print >> f, "   %s" % host_a.os[i]
 
-        table = Table(u"*")
-        for sr_diff in self.script_result_diffs:
-            sr_diff.append_to_port_table(table)
-        if len(table) > 0:
-            print >> f
-            if len(host_b.script_results) == 0:
-                print >> f, u"-Host script results:"
-            elif len(host_a.script_results) == 0:
-                print >> f, u"+Host script results:"
-            else:
-                print >> f, u" Host script results:"
-            print >> f, table
+        print_script_result_diffs_text("Host script results",
+            host_a.script_results, host_b.script_results,
+            self.script_result_diffs)
 
     def to_dom_fragment(self, document):
         host_a = self.host_a
@@ -715,24 +773,10 @@ class HostDiff(object):
         # Host script changes.
         if len(self.script_result_diffs) > 0 or verbose:
             hostscript_elem = document.createElement(u"hostscript")
-            if len(host_a.script_results) == 0 and len(host_b.script_results) == 0:
-                pass
-            elif len(host_b.script_results) == 0:
-                a_elem = document.createElement(u"a")
-                for sr in host_a.script_results:
-                    a_elem.appendChild(sr.to_dom_fragment(document))
-                a_elem.appendChild(hostscript_elem)
-                host_elem.appendChild(a_elem)
-            elif len(host_a.script_results) == 0:
-                b_elem = document.createElement(u"b")
-                for sr in host_b.script_results:
-                    b_elem.appendChild(sr.to_dom_fragment(document))
-                b_elem.appendChild(hostscript_elem)
-                host_elem.appendChild(b_elem)
-            else:
-                for sr_diff in self.script_result_diffs:
-                    hostscript_elem.appendChild(sr_diff.to_dom_fragment(document))
-                host_elem.appendChild(hostscript_elem)
+            host_elem.appendChild(script_result_diffs_to_dom_fragment(
+                hostscript_elem, host_a.script_results,
+                host_b.script_results, self.script_result_diffs,
+                document))
 
         hostdiff_elem.appendChild(host_elem)
 
@@ -1099,24 +1143,27 @@ class NmapContentHandler(xml.sax.handler.ContentHandler):
             self.current_port.service.extrainfo = attrs.get(u"extrainfo")
             self.current_port.service.tunnel = attrs.get(u"tunnel")
         elif name == u"script":
-            assert self.current_host is not None
             result = ScriptResult()
             try:
                 result.id = attrs[u"id"]
             except KeyError:
-                warn(u"%s element of host %s missing the \"id\" attribute; skipping." % (name, self.current_host.format_name()))
+                warn(u"%s element missing the \"id\" attribute; skipping." % name)
                 return
             try:
                 result.output = attrs[u"output"]
             except KeyError:
-                warn(u"%s element of host %s missing the \"output\" attribute; skipping." % (name, self.current_host.format_name()))
+                warn(u"%s element missing the \"output\" attribute; skipping." % name)
                 return
-            if self.parent_element() == u"hostscript":
+            if self.parent_element() == u"prescript":
+                self.scan.pre_script_results.append(result)
+            elif self.parent_element() == u"postscript":
+                self.scan.post_script_results.append(result)
+            elif self.parent_element() == u"hostscript":
                 self.current_host.script_results.append(result)
             elif self.parent_element() == u"port":
                 self.current_port.script_results.append(result)
             else:
-                warn(u"%s element of host %s not inside hostscript or port element; ignoring." % (name, self.current_host.format_name()))
+                warn(u"%s element not inside prescript, postscript, hostscript, or port element; ignoring." % name)
                 return
         elif name == u"osmatch":
             assert self.parent_element() == u"os"
@@ -1193,10 +1240,14 @@ def main():
     filename_a = input_filenames[0]
     filename_b = input_filenames[1]
 
-    scan_a = Scan()
-    scan_a.load_from_file(filename_a)
-    scan_b = Scan()
-    scan_b.load_from_file(filename_b)
+    try:
+        scan_a = Scan()
+        scan_a.load_from_file(filename_a)
+        scan_b = Scan()
+        scan_b.load_from_file(filename_b)
+    except IOError, e:
+        print >> sys.stderr, u"Can't open file: %s" % str(e)
+        sys.exit(EXIT_ERROR)
 
     diff = ScanDiff(scan_a, scan_b)
 
