@@ -1,6 +1,15 @@
+#include "nmap.h"
+#include "nmap_error.h"
+#include "portlist.h"
+#include "nsock.h"
+#include "NmapOps.h"
+#include "timing.h"
+#include "Target.h"
+#include "nmap_tty.h"
+#include "xml.h"
 
 #include "nse_main.h"
-
+#include "nse_utility.h"
 #include "nse_fs.h"
 #include "nse_nsock.h"
 #include "nse_nmaplib.h"
@@ -10,18 +19,14 @@
 #include "nse_openssl.h"
 #include "nse_debug.h"
 
-#include "nmap.h"
-#include "nmap_error.h"
-#include "portlist.h"
-#include "nsock.h"
-#include "NmapOps.h"
-#include "timing.h"
-#include "Target.h"
-#include "nmap_tty.h"
-
 #define NSE_MAIN "NSE_MAIN" /* the main function */
 #define NSE_TRACEBACK "NSE_TRACEBACK"
 
+/* Script Scan phases */
+#define NSE_PRE_SCAN  "NSE_PRE_SCAN"
+#define NSE_SCAN      "NSE_SCAN"
+#define NSE_POST_SCAN "NSE_POST_SCAN"
+
 /* These are indices into the registry, for data shared with nse_main.lua. The
    definitions here must match those in nse_main.lua. */
 #define NSE_YIELD "NSE_YIELD"
@@ -31,10 +36,15 @@
 #define NSE_SELECTED_BY_NAME "NSE_SELECTED_BY_NAME"
 #define NSE_CURRENT_HOSTS "NSE_CURRENT_HOSTS"
 
-#define MAX_FILENAME_LEN 4096
+#ifndef MAXPATHLEN
+#  define MAXPATHLEN 2048
+#endif
 
 extern NmapOps o;
 
+/* global object to store Pre-Scan and Post-Scan script results */
+static ScriptResults script_scan_results;
+
 static int timedOut (lua_State *L)
 {
   Target *target = get_target(L, 1);
@@ -99,6 +109,15 @@ static int ports (lua_State *L)
   return 3;
 }
 
+static int script_set_output (lua_State *L)
+{
+  ScriptResult sr;
+  sr.set_id(luaL_checkstring(L, 1));
+  sr.set_output(luaL_checkstring(L, 2));
+  script_scan_results.push_back(sr);
+  return 0;
+}
+
 static int host_set_output (lua_State *L)
 {
   ScriptResult sr;
@@ -111,7 +130,8 @@ static int host_set_output (lua_State *L)
 
 static int port_set_output (lua_State *L)
 {
-  Port port, *p;
+  Port *p;
+  Port port;
   ScriptResult sr;
   Target *target = get_target(L, 1);
   p = get_port(L, target, &port, 2);
@@ -123,39 +143,6 @@ static int port_set_output (lua_State *L)
   return 0;
 }
 
-static int fetchfile_absolute (lua_State *L)
-{
-  char path[MAX_FILENAME_LEN];
-  switch (nse_fetchfile_absolute(path, sizeof(path), luaL_checkstring(L, 1)))
-  {
-    case 0: // no such path
-      lua_pushnil(L);
-      lua_pushfstring(L, "no path to file/directory: %s", lua_tostring(L, 1));
-      break;
-    case 1: // file returned
-      lua_pushliteral(L, "file");
-      lua_pushstring(L, path);
-      break;
-    case 2: // directory returned
-      lua_pushliteral(L, "directory");
-      lua_pushstring(L, path);
-      break;
-    default:
-      return luaL_error(L, "nse_fetchfile_absolute returned bad code");
-  }
-  return 2;
-}
-
-static int dump_dir (lua_State *L)
-{
-  luaL_checkstring(L, 1);
-  lua_pushcclosure(L, nse_scandir, 0);
-  lua_pushvalue(L, 1);
-  lua_pushinteger(L, NSE_FILES);
-  lua_call(L, 2, 1);
-  return 1;
-}
-
 /* This must call the l_nsock_loop function defined in nse_nsock.cc.
  * That closure is created in luaopen_nsock in order to allow
  * l_nsock_loop to have access to the nsock library environment.
@@ -175,12 +162,6 @@ static int key_was_pressed (lua_State *L)
   return 1;
 }
 
-static int updatedb (lua_State *L)
-{
-  lua_pushboolean(L, script_updatedb());
-  return 1;
-}
-
 static int scp (lua_State *L)
 {
   static const char * const ops[] = {"printStats", "printStatsIfNecessary",
@@ -213,35 +194,101 @@ static int scan_progress_meter (lua_State *L)
   return 1;
 }
 
+/* This is like nmap.log_write, but doesn't append "NSE:" to the beginning of
+   messages. It is only used internally by nse_main.lua and is not available to
+   scripts. */
+static int l_log_write(lua_State *L)
+{
+  static const char *const ops[] = {"stdout", "stderr", NULL};
+  static const int logs[] = {LOG_STDOUT, LOG_STDERR};
+  int log = logs[luaL_checkoption(L, 1, NULL, ops)];
+  log_write(log, "%s", luaL_checkstring(L, 2));
+  return 0;
+}
+
+static int l_xml_start_tag(lua_State *L)
+{
+  const char *name;
+
+  name = luaL_checkstring(L, 1);
+  xml_open_start_tag(name);
+
+  if (lua_isnoneornil(L, 2)) {
+    lua_newtable(L);
+    lua_replace(L, 2);
+  }
+
+  lua_pushnil(L);
+  while (lua_next(L, 2)) {
+    xml_attribute(luaL_checkstring(L, -2), "%s", luaL_checkstring(L, -1));
+    lua_pop(L, 1);
+  }
+
+  xml_close_start_tag();
+
+  return 0;
+}
+
+static int l_xml_end_tag(lua_State *L)
+{
+  xml_end_tag();
+
+  return 0;
+}
+
+static int l_xml_write_escaped(lua_State *L)
+{
+  const char *text;
+
+  text = luaL_checkstring(L, 1);
+  xml_write_escaped(text);
+
+  return 0;
+}
+
+static int l_xml_newline(lua_State *L)
+{
+  xml_newline();
+
+  return 0;
+}
+
 static void open_cnse (lua_State *L)
 {
   static const luaL_Reg nse[] = {
     {"fetchfile_absolute", fetchfile_absolute},
-    {"dump_dir", dump_dir},
+    {"dir", nse_readdir},
     {"nsock_loop", nsock_loop},
     {"key_was_pressed", key_was_pressed},
-    {"updatedb", updatedb},
     {"scan_progress_meter", scan_progress_meter},
     {"timedOut", timedOut},
     {"startTimeOutClock", startTimeOutClock},
     {"stopTimeOutClock", stopTimeOutClock},
     {"ports", ports},
+    {"script_set_output", script_set_output},
     {"host_set_output", host_set_output},
     {"port_set_output", port_set_output},
+    {"log_write", l_log_write},
+    {"xml_start_tag", l_xml_start_tag},
+    {"xml_end_tag", l_xml_end_tag},
+    {"xml_write_escaped", l_xml_write_escaped},
+    {"xml_newline", l_xml_newline},
     {NULL, NULL}
   };
 
+  /* create dir metatable */
+  luaopen_fs(L);
+
   lua_newtable(L);
   luaL_register(L, NULL, nse);
   /* Add some other fields */
-  lua_pushboolean(L, o.script == 1); /* default scripts if none enumerated? */
-  lua_setfield(L, -2, "default");
-  lua_pushboolean(L, o.scriptversion == 1);
-  lua_setfield(L, -2, "scriptversion");
-  lua_pushliteral(L, SCRIPT_ENGINE_LUA_DIR SCRIPT_ENGINE_DATABASE);
-  lua_setfield(L, -2, "script_dbpath");
-  lua_pushstring(L, o.scriptargs);
-  lua_setfield(L, -2, "scriptargs");
+  setbfield(L, -1, "default", o.script == 1);
+  setbfield(L, -1, "scriptversion", o.scriptversion == 1);
+  setbfield(L, -1, "scriptupdatedb", o.scriptupdatedb == 1);
+  setbfield(L, -1, "scripthelp", o.scripthelp);
+  setsfield(L, -1, "script_dbpath", SCRIPT_ENGINE_DATABASE);
+  setsfield(L, -1, "scriptargs", o.scriptargs);
+  setsfield(L, -1, "NMAP_URL", NMAP_URL);
 }
 
 void ScriptResult::set_output (const char *out)
@@ -249,9 +296,9 @@ void ScriptResult::set_output (const char *out)
   output = std::string(out);
 }
 
-std::string ScriptResult::get_output (void) const
+const char *ScriptResult::get_output (void) const
 {
-  return output;
+  return output.c_str();
 }
 
 void ScriptResult::set_id (const char *ident)
@@ -259,9 +306,14 @@ void ScriptResult::set_id (const char *ident)
   id = std::string(ident);
 }
 
-std::string ScriptResult::get_id (void) const
+const char *ScriptResult::get_id (void) const
 {
-  return id;
+  return id.c_str();
+}
+
+ScriptResults *get_script_scan_results_obj (void)
+{
+  return &script_scan_results;
 }
 
 /* int panic (lua_State *L)
@@ -279,13 +331,13 @@ static void set_nmap_libraries (lua_State *L)
 {
   static const luaL_Reg libs[] = {
     {NSE_PCRELIBNAME, luaopen_pcrelib}, // pcre library
-    {"nmap", luaopen_nmap}, // nmap bindings
+    {NSE_NMAPLIBNAME, luaopen_nmap}, // nmap bindings
     {NSE_BINLIBNAME, luaopen_binlib},
     {BITLIBNAME, luaopen_bit}, // bit library
 #ifdef HAVE_OPENSSL
     {OPENSSLLIBNAME, luaopen_openssl}, // openssl bindings
 #endif
-    {"stdnse.c", luaopen_stdnse_c},
+    {NSE_STDNSELIBNAME, luaopen_stdnse_c},
     {NULL, NULL}
   };
 
@@ -306,78 +358,9 @@ static void set_nmap_libraries (lua_State *L)
   lua_pop(L, 3); /* require, package, package.preload */
 }
 
-int script_updatedb (void)
-{
-  static const char load_db[] = 
-    "local nse = ...\n"
-    "local _G, assert, ipairs, loadfile, setfenv, setmetatable, rawget, type ="
-    "      _G, assert, ipairs, loadfile, setfenv, setmetatable, rawget, type\n"
-    "local lower, match, create, resume, open = \n"
-    "  string.lower, string.match, coroutine.create, coroutine.resume,"
-    "  io.open\n"
-    /* set the package.path */
-    "local t, path = assert(nse.fetchfile_absolute('nselib/'))\n"
-    "assert(t == 'directory', 'could not locate nselib directory!')\n"
-    "package.path = package.path..';'..path..'?.lua'\n"
-    /* fetch the scripts directory */
-    "local t, path = nse.fetchfile_absolute('scripts/')\n"
-    "assert(t == 'directory', 'could not locate scripts directory')\n"
-    "local db = assert(open(path..'script.db', 'w'),\n"
-    "  'could not open database for writing')\n"
-    /* dump the scripts/categories */
-    "local scripts = nse.dump_dir(path)\n"
-    "table.sort(scripts)\n"
-    "for i, script in ipairs(scripts) do\n"
-    "  local env = setmetatable({}, {__index = _G})\n"
-    "  local thread = create(setfenv(assert(loadfile(script)), env))\n"
-    "  assert(resume(thread))\n"
-    "  local categories = rawget(env, 'categories')\n"
-    "  assert(type(categories) == 'table', script.."
-    "    ' categories field is not a table')\n"
-    "  local basename = assert(match(script, '[/\\\\]?([^/\\\\]-%.nse)$'))\n"
-    "  table.sort(categories)\n"
-    "  db:write('Entry { filename = \"', basename, '\", categories = {')\n"
-    "  for j, category in ipairs(categories) do\n"
-    "    db:write(' \"', lower(category), '\",')\n"
-    "  end\n"
-    "  db:write(' } }\\n')\n"
-    "end\n"
-    "db:close()\n";
-  int status = 1;
-  lua_State *L;
-
-  log_write(LOG_STDOUT, "%s: Updating rule database.\n", SCRIPT_ENGINE);
-
-  L = luaL_newstate();
-  if (L == NULL)
-    fatal("%s: error opening lua for database update\n", SCRIPT_ENGINE);
-  lua_atpanic(L, panic); /* we let Lua panic if memory error */
-  luaL_openlibs(L);
-  set_nmap_libraries(L);
-  
-  lua_settop(L, 0); // safety, is 0 anyway
-  lua_getglobal(L, "debug");
-  lua_getfield(L, -1, "traceback");
-  lua_replace(L, -2);
-
-  if (luaL_loadstring(L, load_db) != 0)
-    fatal("%s: loading load_db failed %s", SCRIPT_ENGINE, lua_tostring(L, -1));
-  open_cnse(L);
-  if (lua_pcall(L, 1, 0, 1) != 0)
-  {
-    error("%s: error while updating Script Database:\n%s\n",
-        SCRIPT_ENGINE, lua_tostring(L, -1));
-    status = 0;
-  }
-  else
-    log_write(LOG_STDOUT, "NSE script database updated successfully.\n");
-  lua_close(L);
-  return status;
-}
-
 static int init_main (lua_State *L)
 {
-  char path[MAX_FILENAME_LEN];
+  char path[MAXPATHLEN];
   std::vector<std::string> *rules = (std::vector<std::string> *)
       lua_touserdata(L, 1);
 
@@ -397,7 +380,7 @@ static int init_main (lua_State *L)
   lua_setfield(L, LUA_REGISTRYINDEX, NSE_TRACEBACK); /* save copy */
 
   /* Load main Lua code, stack position 2 */
-  if (nmap_fetchfile(path, MAX_FILENAME_LEN, "nse_main.lua") != 1)
+  if (nmap_fetchfile(path, sizeof(path), "nse_main.lua") != 1)
     luaL_error(L, "could not locate nse_main.lua");
   if (luaL_loadfile(L, path) != 0)
     luaL_error(L, "could not load nse_main.lua: %s", lua_tostring(L, -1));
@@ -465,11 +448,29 @@ static int run_main (lua_State *L)
   }
   lua_pop(L, 1); /* pop NSE_CURRENT_HOSTS */
 
-  if (lua_pcall(L, 1, 0, 1) != 0) lua_error(L); /* we wanted a traceback */
+  /* push script scan type phase */
+  switch (o.current_scantype)
+  {
+    case SCRIPT_PRE_SCAN:
+      lua_pushstring(L, NSE_PRE_SCAN);
+      break;
+    case SCRIPT_SCAN:
+      lua_pushstring(L, NSE_SCAN);
+      break;
+    case SCRIPT_POST_SCAN:
+      lua_pushstring(L, NSE_POST_SCAN);
+      break;
+    default:
+      fatal("%s: failed to set the script scan phase.\n", SCRIPT_ENGINE);
+  }
 
+  if (lua_pcall(L, 2, 0, 1) != 0) lua_error(L); /* we wanted a traceback */
   return 0;
 }
 
+/* Lua 5.2 compatibility macro */
+#define lua_yieldk(L,n,ctx,k)  lua_yield(L,n)
+
 /* int nse_yield (lua_State *L)                            [-?, +?, e]
  *
  * This function will yield the running thread back to NSE, even across script
@@ -477,12 +478,12 @@ static int run_main (lua_State *L)
  * correct and only way to call is as a tail call:
  *   return nse_yield(L);
  */
-int nse_yield (lua_State *L)
+int nse_yield (lua_State *L, int ctx, lua_CFunction k)
 {
   lua_getfield(L, LUA_REGISTRYINDEX, NSE_YIELD);
   lua_pushthread(L);
   lua_call(L, 1, 1); /* returns NSE_YIELD_VALUE */
-  return lua_yield(L, 1); /* yield with NSE_YIELD_VALUE */
+  return lua_yieldk(L, 1, ctx, k); /* yield with NSE_YIELD_VALUE */
 }
 
 /* void nse_restore (lua_State *L, int number)             [-, -, e]
@@ -502,7 +503,7 @@ void nse_restore (lua_State *L, int number)
   /* Call WAITING_TO_RUNNING (defined in nse_main.lua) on the thread and any
      other arguments. */
   if (lua_pcall(L, number+1, 0, 0) != 0)
-    fatal("nse_restore: WAITING_TO_RUNNING error!\n%s", lua_tostring(L, -1));
+    fatal("%s: WAITING_TO_RUNNING error!\n%s", __func__, lua_tostring(L, -1));
 }
 
 /* void nse_destructor (lua_State *L, char what)           [-(1|2), +0, e]
@@ -532,7 +533,7 @@ void nse_destructor (lua_State *L, char what)
     lua_pushnil(L); /* no destructor, we are removing */
   }
   if (lua_pcall(L, 4, 0, 0) != 0)
-    fatal("nse_destructor: NSE_DESTRUCTOR error!\n%s", lua_tostring(L, -1));
+    fatal("%s: NSE_DESTRUCTOR error!\n%s", __func__, lua_tostring(L, -1));
   lua_pop(L, what == 'a' ? 2 : 1);
 }
 
@@ -591,24 +592,36 @@ void open_nse (void)
       fatal("%s: failed to open a Lua state!", SCRIPT_ENGINE);
     lua_atpanic(L_NSE, panic);
 
-    if (lua_cpcall(L_NSE, init_main, (void *) &o.chosenScripts) != 0)
+#if 0
+    /* Lua 5.2 */
+    lua_pushcfunction(L_NSE, init_main);
+    lua_pushlightuserdata(L_NSE, &o.chosenScripts);
+    if (lua_pcall(L_NSE, 1, 0, 0))
+#else
+    if (lua_cpcall(L_NSE, init_main, &o.chosenScripts))
+#endif
       fatal("%s: failed to initialize the script engine:\n%s\n", SCRIPT_ENGINE, 
           lua_tostring(L_NSE, -1));
   }
 }
 
-void script_scan (std::vector<Target *> &targets)
+void script_scan (std::vector<Target *> &targets, stype scantype)
 {
-  o.current_scantype = SCRIPT_SCAN;
+  o.current_scantype = scantype;
 
   assert(L_NSE != NULL);
   lua_settop(L_NSE, 0); /* clear the stack */
 
-  if (lua_cpcall(L_NSE, run_main, (void *) &targets) != 0)
-  {
+#if 0
+  /* Lua 5.2 */
+  lua_pushcfunction(L_NSE, run_main);
+  lua_pushlightuserdata(L_NSE, &targets);
+  if (lua_pcall(L_NSE, 1, 0, 0))
+#else
+  if (lua_cpcall(L_NSE, run_main, &targets))
+#endif
     error("%s: Script Engine Scan Aborted.\nAn error was thrown by the "
           "engine: %s", SCRIPT_ENGINE, lua_tostring(L_NSE, -1));
-  }
 }
 
 void close_nse (void)
