@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2009 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2011 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -17,15 +17,15 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
- *                                                                         * 
+ *                                                                         *
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
- * agreement takes precedence over this comment.                          *
+ * agreement takes precedence over this comment.                           *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
@@ -54,12 +54,12 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock.h 13448 2009-05-29 23:19:07Z david $ */
+/* $Id: nsock.h 21905 2011-01-21 00:04:51Z fyodor $ */
 
 /* Would you like to include pcap support in nsock?
  * Pcap support code is currently unstable, so we give
  * you a choice. In future this #define will be removed.*/
-#define HAVE_PCAP 1
+//#define HAVE_PCAP 1
 
 #ifndef NSOCK_H
 #define NSOCK_H
@@ -67,6 +67,13 @@
 /* Keep assert() defined for security reasons */
 #undef NDEBUG
 
+#ifndef WIN32
+#include "nsock_config.h"
+#else
+#include "nsock_winconfig.h"
+#endif
+
+#include <stdio.h>
 #include <sys/types.h>
 #ifndef WIN32
 #include <sys/socket.h>
@@ -134,9 +141,13 @@ typedef void *nsock_ssl;
    example you could do a series of 15 second runs, allowing you to do
    other stuff between them.  Or you could just schedule a timer to
    call you back every 15 seconds.*/
-enum nsock_loopstatus { NSOCK_LOOP_NOEVENTS = 2, NSOCK_LOOP_TIMEOUT, NSOCK_LOOP_ERROR };
+enum nsock_loopstatus { NSOCK_LOOP_NOEVENTS = 2, NSOCK_LOOP_TIMEOUT, NSOCK_LOOP_ERROR, NSOCK_LOOP_QUIT };
 enum nsock_loopstatus nsock_loop(nsock_pool nsp, int msec_timeout);
 
+/* Calling this function will cause nsock_loop to quit on its next iteration
+   with a return value of NSOCK_LOOP_QUIT. */
+void nsock_loop_quit(nsock_pool nsp);
+
 /* This next function returns the errno style error code -- which is only
    valid if the status is NSOCK_LOOP_ERROR was returned by nsock_loop() */
 int nsp_geterrorcode(nsock_pool nsp);
@@ -163,15 +174,21 @@ void nsp_setud(nsock_pool nsp, void *data);
    to retrieve that data ... */
 void *nsp_getud(nsock_pool nsp);
 
-/* Sets a trace/debug level.  Zero (the default) turns tracing off,
-   while higher numbers are more verbose.  This is generally only used
-   for debugging purposes.  Trace logs are printed to stdout.  The
-   initial value is set in nsp_new().  A level of 1 or 2 is usually
-   sufficient, but 10 will ensure you get everything.  The basetime
-   can be NULL to print trace lines with the current time, otherwise
-   the difference between the current time and basetime will be used
-   (the time program execution starts would be a good candidate) */
-void nsp_settrace(nsock_pool nsp, int tracelevel, const struct timeval *basetime);
+/* Sets a trace/debug level and stream.  A level of 0 (the default)
+   turns tracing off, while higher numbers are more verbose.  If the
+   stream given is NULL, it defaults to stdout.  This is generally only
+   used for debugging purposes. A level of 1 or 2 is usually sufficient,
+   but 10 will ensure you get everything.  The basetime can be NULL to
+   print trace lines with the current time, otherwise the difference
+   between the current time and basetime will be used (the time program
+   execution starts would be a good candidate) */
+void nsp_settrace(nsock_pool nsp, FILE *file, int level, const struct timeval *basetime);
+
+/* Turns on or off broadcast support on new sockets. Default is off 
+   (0, false) set in nsp_new(). Any non-zero (true) value sets 
+   SO_BROADCAST on all new sockets (value of optval will be used directly
+   in the setsockopt() call */
+void nsp_setbroadcast(nsock_pool nsp, int optval);
 
 /* Initializes an Nsock pool to create SSL connections. This sets an internal
    SSL_CTX, which is like a template that sets options for all connections that
@@ -326,6 +343,12 @@ int nsi_getsd(nsock_iod nsiod);
    ids for a given nspool (unless you blow through billions of them). */
 unsigned long nsi_id(nsock_iod nsockiod);
 
+/*Returns Packets received in bytes   */
+unsigned long nsi_get_read_count(nsock_iod nsockiod);
+
+/*Returns Packets sent in bytes   */
+unsigned long nsi_get_write_count(nsock_iod nsockiod);
+
   /* Returns 1 if an NSI is communicating via SSL, 0 otherwise */
 int nsi_checkssl(nsock_iod nsockiod);
 
@@ -347,7 +370,7 @@ int nsi_set_ipoptions(nsock_iod nsi, void *ipopts, size_t ipoptslen);
    communication (or comm. attempt) this nsi has been involved with.
    By "involved" with I mean interactions like establishing (or trying
    to) a connection or sending a UDP datagram through an unconnected
-   nsock_iod.  AF is the address family (AF_INET or AF_INET6), Protocl
+   nsock_iod.  AF is the address family (AF_INET or AF_INET6), Protocol
    is IPPROTO_TCP or IPPROTO_UDP.  Pass NULL for information you do
    not need.  If ANY of the information you requested is not
    available, 0 will be returned and the unavailable sockets are
@@ -361,6 +384,10 @@ int nsi_getlastcommunicationinfo(nsock_iod ms_iod, int *protocol,
 				 int *af, struct sockaddr *local, 
 				 struct sockaddr *remote, size_t socklen);
 
+/* Set the hostname of the remote host, for when that matters. This is
+   currently only used for Server Name Indication in SSL connections. */
+int nsi_set_hostname(nsock_iod nsi, const char *hostname);
+
 /* EVENT CREATION FUNCTIONS -- These functions request asynchronous
    notification of completion of an event.  The handler will never be
    synchronously called back during the event creation call (that
@@ -397,6 +424,9 @@ int nsi_getlastcommunicationinfo(nsock_iod ms_iod, int *protocol,
 
 typedef void (*nsock_ev_handler)(nsock_pool, nsock_event, void *);
 
+/* Initialize an unconnected UDP socket. */
+int nsock_setup_udp(nsock_pool nsp, nsock_iod ms_iod, int af);
+
 /* Request a TCP connection to another system (by IP address).  The
    in_addr is normal network byte order, but the port number should be
    given in HOST BYTE ORDER.  ss should be a sockaddr_storage,
@@ -408,6 +438,17 @@ nsock_event_id nsock_connect_tcp(nsock_pool nsp, nsock_iod nsiod,
 				 void *userdata, struct sockaddr *ss,
 				 size_t sslen, unsigned short port);
 
+/* Request an SCTP association to another system (by IP address).  The
+   in_addr is normal network byte order, but the port number should be
+   given in HOST BYTE ORDER.  ss should be a sockaddr_storage,
+   sockaddr_in6, or sockaddr_in as appropriate (just like what you
+   would pass to connect).  sslen should be the sizeof the structure
+   you are passing in. */
+nsock_event_id nsock_connect_sctp(nsock_pool nsp, nsock_iod nsiod, 
+				 nsock_ev_handler handler, int timeout_msecs, 
+				 void *userdata, struct sockaddr *ss,
+				 size_t sslen, unsigned short port);
+
 /* Request a UDP "connection" to another system (by IP address).  The
    in_addr is normal network byte order, but the port number should be
    given in HOST BYTE ORDER.  Since this is UDP, no packets are
@@ -429,10 +470,10 @@ nsock_event_id nsock_connect_udp(nsock_pool nsp, nsock_iod nsiod,
 				 struct sockaddr *ss, size_t sslen,
 				 unsigned short port);
 
-/* Request an SSL over TCP connection to another system (by IP
+/* Request an SSL over TCP/SCTP connection to another system (by IP
    address).  The in_addr is normal network byte order, but the port
    number should be given in HOST BYTE ORDER.  This function will call
-   back only after it has made the TCP connection AND done the initial
+   back only after it has made the connection AND done the initial
    SSL negotiation.  From that point on, you use the normal read/write
    calls and decryption will happen transparently. ss should be a
    sockaddr_storage, sockaddr_in6, or sockaddr_in as appropriate (just
@@ -441,12 +482,12 @@ nsock_event_id nsock_connect_udp(nsock_pool nsp, nsock_iod nsiod,
 nsock_event_id nsock_connect_ssl(nsock_pool nsp, nsock_iod nsiod, 
 				 nsock_ev_handler handler, int timeout_msecs, 
 				 void *userdata, struct sockaddr *ss, 
-				 size_t sslen, unsigned short port,
+				 size_t sslen, int proto, unsigned short port,
 				 nsock_ssl_session ssl_session);
 
-/* Request ssl connection over already established TCP connection.
+/* Request ssl connection over already established TCP/SCTP connection.
    nsiod must be socket that is already connected to target
-   using nsock_connect_tcp.
+   using nsock_connect_tcp or nsock_connect_sctp.
    All parameters have the same meaning as in 'nsock_connect_ssl' */
 nsock_event_id nsock_reconnect_ssl(nsock_pool nsp, nsock_iod nsiod,
 				   nsock_ev_handler handler, int timeout_msecs,
@@ -479,6 +520,11 @@ nsock_event_id nsock_write(nsock_pool nsp, nsock_iod nsiod,
 		     nsock_ev_handler handler, int timeout_msecs, 
 		     void *userdata, const char *data, int datalen);
 
+nsock_event_id nsock_sendto(nsock_pool ms_pool, nsock_iod ms_iod,
+              nsock_ev_handler handler, int timeout_msecs,
+              void *userdata, struct sockaddr *saddr, size_t sslen,
+              unsigned short port, const char *data, int datalen);
+
 /* Same as nsock_write except you can use a printf-style format and you can only use this for ASCII strings */
 nsock_event_id nsock_printf(nsock_pool nsp, nsock_iod nsiod, 
 		      nsock_ev_handler handler, int timeout_msecs, 
