@@ -6,7 +6,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2009 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2011 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -18,15 +18,15 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
- *                                                                         * 
+ *                                                                         *
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
- * agreement takes precedence over this comment.                          *
+ * agreement takes precedence over this comment.                           *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
@@ -55,7 +55,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_internal.h 13448 2009-05-29 23:19:07Z david $ */
+/* $Id: nsock_internal.h 21905 2011-01-21 00:04:51Z fyodor $ */
 
 #ifndef NSOCK_INTERNAL_H
 #define NSOCK_INTERNAL_H
@@ -99,7 +99,9 @@
 #include <strings.h>
 #endif
 
-#include "nsock_utils.h"
+#ifndef IPPROTO_SCTP
+#define IPPROTO_SCTP 132
+#endif
 
 /*********      STRUCTURES   **************/
 
@@ -158,6 +160,8 @@ struct readinfo {
 };
 
 struct writeinfo {
+  struct sockaddr_storage dest;
+  size_t destlen;
   int written_so_far; /* Number of bytes successfully written */
 };
 
@@ -178,10 +182,17 @@ typedef struct  {
   unsigned long next_iod_serial; /* Serial # of next iod to be created */
   int errnum; /* If nsock_loop() returns NSOCK_LOOP_ERROR, this is where we
 		describe the error (errnum fashion) */
-  int tracelevel; /* Trace/debug level - set by nsp_settrace.  If positive, trace logs are printted to
-		     stdout */
+  int tracelevel; /* Trace/debug level - set by nsp_settrace. If positive,
+                     trace logs are printted to tracefile. */
+  FILE *tracefile;
+  int broadcast; /* If true, new sockets will have SO_BROADCAST set */
   /* This time is subtracted from the current time for trace reports */
   struct timeval tracebasetime; 
+
+  /* If true, exit the next iteration of nsock_loop with a status of
+     NSOCK_LOOP_QUIT. */
+  int quit;
+
 #if HAVE_OPENSSL
   SSL_CTX *sslctx; /* The SSL Context (options and such) */
 #endif
@@ -210,6 +221,7 @@ struct msiod {
      writability. */
   int readsd_count;
   int writesd_count;
+  int readpcapsd_count;
   mspool *nsp; /* The mspool used to create the iod (used for deletion) */
   enum msiod_state state;
   struct sockaddr_storage peer; /* The host and port we are connected to
@@ -229,6 +241,10 @@ struct msiod {
 					     this msiod from the
 					     allocated list when
 					     neccessary */
+
+  /* Used for SSL Server Name Indication. */
+  char *hostname;
+
 #if HAVE_OPENSSL
   SSL *ssl; /* An SSL connection (or NULL if none) */
   SSL_SESSION *ssl_session; /* SSL SESSION ID (or NULL if none) */
@@ -237,6 +253,9 @@ struct msiod {
 #endif
   unsigned long id; /* Every iod has an id which is always unique for the
 		       same nspool (unless you create billions of them) */
+
+  unsigned long read_count;  /* No. of bytes read  from the sd*/ 
+  unsigned long write_count; /* No. of bytes written to the sd */
   void *userdata;
 
   /* IP options to set on socket before connect() */
@@ -323,6 +342,10 @@ void msevent_delete(mspool *nsp, msevent *nse);
    timeout value, etc. */
 void nsp_add_event(mspool *nsp, msevent *nse);
 
+void nsock_connect_internal(mspool *ms, msevent *nse, int proto,
+			    struct sockaddr_storage *ss, size_t sslen,
+			    unsigned short port);
+
 /* Comments on using the following handle_*_result functions are available
    in nsock_core.c */
 /* handle_connect_results assumes that select or poll have already
