@@ -1,23 +1,90 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2005 Insecure.Com LLC.
-#
-# Author: Adriano Monteiro Marques <py.adriano@gmail.com>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+# ***********************IMPORTANT NMAP LICENSE TERMS************************
+# *                                                                         *
+# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+# * also a registered trademark of Insecure.Com LLC.  This program is free  *
+# * software; you may redistribute and/or modify it under the terms of the  *
+# * GNU General Public License as published by the Free Software            *
+# * Foundation; Version 2 with the clarifications and exceptions described  *
+# * below.  This guarantees your right to use, modify, and redistribute     *
+# * this software under certain conditions.  If you wish to embed Nmap      *
+# * technology into proprietary software, we sell alternative licenses      *
+# * (contact sales@insecure.com).  Dozens of software vendors already       *
+# * license Nmap technology such as host discovery, port scanning, OS       *
+# * detection, and version detection.                                       *
+# *                                                                         *
+# * Note that the GPL places important restrictions on "derived works", yet *
+# * it does not provide a detailed definition of that term.  To avoid       *
+# * misunderstandings, we consider an application to constitute a           *
+# * "derivative work" for the purpose of this license if it does any of the *
+# * following:                                                              *
+# * o Integrates source code from Nmap                                      *
+# * o Reads or includes Nmap copyrighted data files, such as                *
+# *   nmap-os-db or nmap-service-probes.                                    *
+# * o Executes Nmap and parses the results (as opposed to typical shell or  *
+# *   execution-menu apps, which simply display raw Nmap output and so are  *
+# *   not derivative works.)                                                *
+# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+# *   installer, such as those produced by InstallShield.                   *
+# * o Links to a library or executes a program that does any of the above   *
+# *                                                                         *
+# * The term "Nmap" should be taken to also include any portions or derived *
+# * works of Nmap.  This list is not exclusive, but is meant to clarify our *
+# * interpretation of derived works with some common examples.  Our         *
+# * interpretation applies only to Nmap--we don't speak for other people's  *
+# * GPL works.                                                              *
+# *                                                                         *
+# * If you have any questions about the GPL licensing restrictions on using *
+# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
+# * we also offer alternative license to integrate Nmap into proprietary    *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing for priority support and updates as well as helping to     *
+# * fund the continued development of Nmap technology.  Please email        *
+# * sales@insecure.com for further information.                             *
+# *                                                                         *
+# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+# * permission to link the code of this program with any version of the     *
+# * OpenSSL library which is distributed under a license identical to that  *
+# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+# * linked combinations including the two. You must obey the GNU GPL in all *
+# * respects for all of the code used other than OpenSSL.  If you modify    *
+# * this file, you may extend this exception to your version of the file,   *
+# * but you are not obligated to do so.                                     *
+# *                                                                         *
+# * If you received these files with a written license agreement or         *
+# * contract stating terms other than the terms above, then that            *
+# * alternative license agreement takes precedence over these comments.     *
+# *                                                                         *
+# * Source is provided to this software because we believe users have a     *
+# * right to know exactly what a program is going to do before they run it. *
+# * This also allows you to audit the software for security holes (none     *
+# * have been found so far).                                                *
+# *                                                                         *
+# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
+# * and add new features.  You are highly encouraged to send your changes   *
+# * to nmap-dev@insecure.org for possible incorporation into the main       *
+# * distribution.  By sending these changes to Fyodor or one of the         *
+# * Insecure.Org development mailing lists, it is assumed that you are      *
+# * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because the *
+# * inability to relicense code has caused devastating problems for other   *
+# * Free Software projects (such as KDE and NASM).  We also occasionally    *
+# * relicense the code to third parties as discussed above.  If you wish to *
+# * specify special license conditions of your contributions, just say so   *
+# * when you send them.                                                     *
+# *                                                                         *
+# * This program is distributed in the hope that it will be useful, but     *
+# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
+# * General Public License v2.0 for more details at                         *
+# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+# * included with Nmap.                                                     *
+# *                                                                         *
+# ***************************************************************************/
 
 import errno
 import sys
@@ -59,7 +126,7 @@ def mo_find(result, dirname, fnames):
         p = os.path.join(dirname, f)
         if os.path.isfile(p) and f.endswith(".mo"):
             files.append(p)
-        
+
     if files:
         result.append((dirname, files))
 
@@ -76,7 +143,7 @@ data_files = [ (pixmaps_dir, glob(os.path.join(pixmaps_dir, '*.gif')) +
                             [os.path.join(config_dir, 'scan_profile.usp')] +
                             [os.path.join(config_dir, APP_NAME + '_version')]),
 
-               (misc_dir, glob(os.path.join(misc_dir, '*.xml'))), 
+               (misc_dir, glob(os.path.join(misc_dir, '*.xml'))),
 
                (docs_dir, [os.path.join(docs_dir, 'help.html')])]
 
@@ -124,6 +191,13 @@ def path_strip_prefix(path, prefix):
 # Distutils subclasses
 
 class my_install(install):
+    def finalize_options(self):
+        # Ubuntu's python2.6-2.6.4-0ubuntu3 package changes sys.prefix in
+        # install.finalize_options when sys.prefix is "/usr/local" (our
+        # default). Because we need the unchanged value later, remember it here.
+        self.saved_prefix = self.prefix
+        install.finalize_options(self)
+
     def run(self):
         install.run(self)
 
@@ -239,19 +313,18 @@ for dir in dirs:
         if self.root is not None:
             modules = path_strip_prefix(modules, self.root)
 
-        re_sys = re.compile("^import sys$")
-
         ufile = open(app_file_name, "r")
         ucontent = ufile.readlines()
         ufile.close()
 
-        uline = None
-        for line in xrange(len(ucontent)):
-            if re_sys.match(ucontent[line]):
-                uline = line + 1
+        # Insert our custom import after the first non-comment line.
+        re_sys = re.compile("^#")
+        uline = 0
+        for line in ucontent:
+            if not re_sys.match(line):
                 break
-
-        ucontent.insert(uline, "sys.path.append(%s)\n" % repr(modules))
+            uline += 1
+        ucontent.insert(uline, "\nimport sys\nsys.path.append(%s)\n" % repr(modules))
 
         ufile = open(app_file_name, "w")
         ufile.writelines(ucontent)
@@ -278,11 +351,13 @@ for dir in dirs:
 
     def fix_paths(self):
         """Replace some hardcoded paths to match where files were installed."""
-        interesting_paths = {"CONFIG_DIR": os.path.join(self.prefix, config_dir),
-                             "DOCS_DIR": os.path.join(self.prefix, docs_dir),
-                             "LOCALE_DIR": os.path.join(self.prefix, locale_dir),
-                             "MISC_DIR": os.path.join(self.prefix, misc_dir),
-                             "PIXMAPS_DIR": os.path.join(self.prefix, pixmaps_dir)}
+        interesting_paths = {"CONFIG_DIR": os.path.join(self.saved_prefix, config_dir),
+                             "DOCS_DIR": os.path.join(self.saved_prefix, docs_dir),
+                             "LOCALE_DIR": os.path.join(self.saved_prefix, locale_dir),
+                             "MISC_DIR": os.path.join(self.saved_prefix, misc_dir),
+                             "PIXMAPS_DIR": os.path.join(self.saved_prefix, pixmaps_dir),
+                             # See $(nmapdatadir) in nmap/Makefile.in.
+                             "NMAPDATADIR": os.path.join(self.saved_prefix, "share", "nmap"),}
 
         # Find and read the Paths.py file.
         pcontent = ""
@@ -310,8 +385,8 @@ for dir in dirs:
         # Rewrite the zenmap.desktop and zenmap-root.desktop files to point to
         # the installed locations of the su-to-zenmap.sh script and application
         # icon.
-        su_filename = os.path.join(self.prefix, data_dir, "su-to-zenmap.sh")
-        icon_filename = os.path.join(self.prefix, pixmaps_dir, "zenmap.png")
+        su_filename = os.path.join(self.saved_prefix, data_dir, "su-to-zenmap.sh")
+        icon_filename = os.path.join(self.saved_prefix, pixmaps_dir, "zenmap.png")
 
         desktop_filename = None
         root_desktop_filename = None
@@ -420,13 +495,11 @@ class my_uninstall(Command):
 # the operations.
 COMMON_SETUP_ARGS = {
     'name': APP_NAME,
-    'license': 'GNU GPL (version 2 or later)',
+    'license': 'Nmap License (http://nmap.org/book/man-legal.html)',
     'url': APP_WEB_SITE,
     'download_url': APP_DOWNLOAD_SITE,
-    'author': 'Adriano Monteiro & Cleber Rodrigues',
-    'author_email': 'py.adriano@gmail.com, cleber@globalred.com.br',
-    'maintainer': 'Adriano Monteiro',
-    'maintainer_email': 'py.adriano@gmail.com',
+    'author': 'Nmap Project',
+    'maintainer': 'Nmap Project',
     'description': "%s frontend and results viewer" % NMAP_DISPLAY_NAME,
     'long_description': """\
 %s is an %s frontend \
@@ -457,20 +530,12 @@ if 'py2exe' in sys.argv:
         'console': [{"script": "../ndiff/ndiff", "description": "Nmap scan comparison tool"}],
         'options': {"py2exe": {
             "compressed": 1,
-            "optimize":2,
-            "packages":"encodings",
-            "includes" : "\
-pango,\
-atk,\
-gobject,\
-pickle,\
-bz2,\
-encodings,\
-encodings.*,\
-cairo,\
-pangocairo,\
-atk\
-"}}
+            "optimize": 2,
+            "packages": ["encodings"],
+            "includes": ["pango", "atk", "gobject", "gio", "pickle", "bz2", "encodings", "encodings.*", "cairo", "pangocairo"],
+            "dll_excludes": ["USP10.dll", "NSI.dll", "MSIMG32.dll", "DNSAPI.dll"]
+            }
+        }
     }
 
     setup_args.update(WINDOWS_SETUP_ARGS)
