@@ -3,7 +3,7 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
+# * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
 # * also a registered trademark of Insecure.Com LLC.  This program is free  *
 # * software; you may redistribute and/or modify it under the terms of the  *
 # * GNU General Public License as published by the Free Software            *
@@ -25,7 +25,7 @@
 # *   nmap-os-db or nmap-service-probes.                                    *
 # * o Executes Nmap and parses the results (as opposed to typical shell or  *
 # *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                * 
+# *   not derivative works.)                                                *
 # * o Integrates/includes/aggregates Nmap into a proprietary executable     *
 # *   installer, such as those produced by InstallShield.                   *
 # * o Links to a library or executes a program that does any of the above   *
@@ -48,8 +48,8 @@
 # * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included COPYING.OpenSSL file, and distribute linked      *
-# * combinations including the two. You must obey the GNU GPL in all        *
+# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+# * linked combinations including the two. You must obey the GNU GPL in all *
 # * respects for all of the code used other than OpenSSL.  If you modify    *
 # * this file, you may extend this exception to your version of the file,   *
 # * but you are not obligated to do so.                                     *
@@ -99,9 +99,10 @@ from zenmapGUI.FileChoosers import AllFilesFileChooserDialog
 from zenmapGUI.ProfileHelp import ProfileHelp
 
 from zenmapCore.Paths import Path
-from zenmapCore.NmapOptions import NmapOptions
+from zenmapCore.NmapOptions import NmapOptions, split_quoted, join_quoted
 import zenmapCore.I18N
 from zenmapCore.UmitLogging import log
+from zenmapGUI.ScriptInterface import *
 
 def get_option_check_auxiliary_widget(option, ops, check):
     if option in ("-sI", "-b", "--script", "--script-args", "--exclude", "-p",
@@ -115,7 +116,7 @@ def get_option_check_auxiliary_widget(option, ops, check):
         return OptionLevel(option, ops, check)
     elif option in ("--excludefile", "-iL"):
         return OptionFile(option, ops, check)
-    elif option in ("-A", "-O", "-sV", "-n", "-6", "-PN", "-PE", "-PP", "-PM",
+    elif option in ("-A", "-O", "-sV", "-n", "-6", "-Pn", "-PE", "-PP", "-PM",
         "-PB", "-sC", "--script-trace", "-F", "-f", "--packet-trace", "-r",
         "--traceroute"):
         return None
@@ -270,20 +271,20 @@ class TargetEntry(gtk.Entry):
         self.ops.target_specs = self.get_targets()
 
     def get_targets(self):
-        return self.get_text().decode("UTF-8").split()
+        return split_quoted(self.get_text().decode("UTF-8"))
 
 class OptionTab(object):
-    def __init__(self, root_tab, ops, update_command, update_help):
+    def __init__(self, root_tab, ops, update_command, help_buf):
         actions = {'target':self.__parse_target,
                    'option_list':self.__parse_option_list,
                    'option_check':self.__parse_option_check}
 
         self.ops = ops
         self.update_command = update_command
-        self.update_help = update_help
+        self.help_buf = help_buf
 
         self.profilehelp = ProfileHelp()
-
+        self.notscripttab = False  # assume every tab is scripting tab
         self.widgets_list = []
         for option_element in root_tab.childNodes:
             try:option_element.tagName
@@ -295,7 +296,7 @@ class OptionTab(object):
                     self.widgets_list.append(widget)
 
     def __parse_target(self, target_element):
-        label = target_element.getAttribute(u'label')
+        label = _(target_element.getAttribute(u'label'))
         label_widget = HIGEntryLabel(label)
         target_widget = TargetEntry(self.ops)
         target_widget.connect("changed", self.update_target)
@@ -303,36 +304,36 @@ class OptionTab(object):
 
     def __parse_option_list(self, option_list_element):
         children = option_list_element.getElementsByTagName(u'option')
-        
-        label_widget = HIGEntryLabel(option_list_element.getAttribute(u'label'))
+
+        label_widget = HIGEntryLabel(_(option_list_element.getAttribute(u'label')))
         option_list_widget = OptionList(self.ops)
-        
+
         for child in children:
             option = child.getAttribute(u'option')
             argument = child.getAttribute(u'argument')
-            label = child.getAttribute(u'label')
+            label = _(child.getAttribute(u'label'))
             option_list_widget.append(option, argument, label)
             self.profilehelp.add_label(option, label)
-            self.profilehelp.add_shortdesc(option, child.getAttribute(u'short_desc'))
+            self.profilehelp.add_shortdesc(option, _(child.getAttribute(u'short_desc')))
             self.profilehelp.add_example(option, child.getAttribute(u'example'))
 
         option_list_widget.update()
 
         option_list_widget.connect("changed", self.update_list_option)
-                
+
         return label_widget, option_list_widget
-    
+
     def __parse_option_check(self, option_check):
         arg_type = option_check.getAttribute(u'arg_type')
         option = option_check.getAttribute(u'option')
-        label = option_check.getAttribute(u'label')
-        short_desc = option_check.getAttribute(u'short_desc')
+        label = _(option_check.getAttribute(u'label'))
+        short_desc = _(option_check.getAttribute(u'short_desc'))
         example = option_check.getAttribute(u'example')
 
         self.profilehelp.add_label(option, label)
         self.profilehelp.add_shortdesc(option, short_desc)
         self.profilehelp.add_example(option, example)
-        
+
         check = OptionCheck(option, label)
         auxiliary_widget = get_option_check_auxiliary_widget(option, self.ops, check)
         if auxiliary_widget is not None:
@@ -343,9 +344,9 @@ class OptionTab(object):
 
         check.connect('toggled', self.update_check, auxiliary_widget)
         check.connect('enter-notify-event', self.enter_notify_event_cb, option)
-        
+
         return check, auxiliary_widget
-        
+
     def fill_table(self, table, expand_fill = True):
         yopt = (0, gtk.EXPAND | gtk.FILL)[expand_fill]
         for y, widget in enumerate(self.widgets_list):
@@ -380,7 +381,7 @@ class OptionTab(object):
     def update_list_option(self, widget):
         if widget.last_selected:
             self.ops[widget.last_selected] = None
-        
+
         opt, arg, label = widget.list[widget.get_active()]
         if opt:
             if arg:
@@ -393,7 +394,7 @@ class OptionTab(object):
         self.show_help_for_option(opt)
 
         self.update_command()
-        
+
     def show_help_for_option(self, option):
         self.profilehelp.handler(option)
         text = ""
@@ -406,13 +407,13 @@ class OptionTab(object):
             if self.profilehelp.get_example():
                 text += "\n\nExample input:\n"
                 text += self.profilehelp.get_example()
-        self.update_help(text)
+        self.help_buf.set_text(text)
 
     def enter_notify_event_cb(self, event, widget, option):
         self.show_help_for_option(option)
 
 class OptionBuilder(object):
-    def __init__(self, xml_file, ops, update_func, update_help):
+    def __init__(self, xml_file, ops, update_func, help_buf):
         """
         xml_file is a UI description xml-file
         ops is an NmapOptions instance
@@ -423,17 +424,18 @@ class OptionBuilder(object):
         xml_desc.close()
 
         self.ops = ops
-        self.update_help = update_help 
+        self.help_buf = help_buf
         self.update_func = update_func
-        
+
         self.root_tag = "interface"
-        
+
+
         self.xml = self.xml.getElementsByTagName(self.root_tag)[0]
-        
+
         self.groups = self.__parse_groups()
         self.section_names = self.__parse_section_names()
         self.tabs = self.__parse_tabs()
-    
+
     def update(self):
         for tab in self.tabs.values():
             tab.update()
@@ -444,7 +446,7 @@ class OptionBuilder(object):
             grp = self.xml.getElementsByTagName(group)[0]
             dic[group] = grp.getAttribute(u'label')
         return dic
-    
+
     def __parse_groups(self):
         return [g_name.getAttribute(u'name') for g_name in \
                   self.xml.getElementsByTagName(u'groups')[0].\
@@ -453,8 +455,12 @@ class OptionBuilder(object):
     def __parse_tabs(self):
         dic = {}
         for tab_name in self.groups:
-            dic[tab_name] = OptionTab(self.xml.getElementsByTagName(tab_name)[0],
-                                      self.ops, self.update_func, self.update_help)
+            if tab_name != "Scripting":
+                dic[tab_name] = OptionTab(self.xml.getElementsByTagName(tab_name)[0],
+                                      self.ops, self.update_func, self.help_buf)
+                dic[tab_name].notscripttab = True
+            else:
+                dic[tab_name] =ScriptInterface(None,self.ops, self.update_func, self.help_buf)
         return dic
 
 
@@ -468,10 +474,10 @@ class OptionList(gtk.ComboBox):
         cell = gtk.CellRendererText()
         self.pack_start(cell, True)
         self.add_attribute(cell, 'text', 2)
-        
+
         self.last_selected = None
         self.options = []
-    
+
     def update(self):
         selected = 0
         for i, row in enumerate(self.list):
@@ -490,7 +496,7 @@ class OptionList(gtk.ComboBox):
                 ops[option] = argument
             else:
                 ops[option] = True
-            opt += " (%s)" % ops.render_string()
+            opt += " (%s)" % join_quoted(ops.render()[1:])
 
         self.list.append([option, argument, opt])
         self.options.append(option)
@@ -502,5 +508,5 @@ class OptionCheck(gtk.CheckButton):
             opt += " (%s)" % option
 
         gtk.CheckButton.__init__(self, opt, use_underline=False)
-        
+
         self.option = option
